//////////////////////////////////////////////////////////////////////
// toolhelp.cpp
// -------------------------------------------------------------------
//
// <thohell@home.se>
//////////////////////////////////////////////////////////////////////
#define THIS_IS_SERVER
#include "..\D2Hackit.h"

//
//	Initialize commands
GAMECOMMANDSTRUCT GameCommands[] = {
	{
		"help",
		GameCommandLineHelp,
		"c4.helpc0 List all commands available in the server dll\n"
		"c4.<command> helpc0 Shows detailed help for specified command.\n"
		"c4.<module> helpc0 Shows help for specified module."
	},
	{
		"load",
		GameCommandLineLoad,
		"c4.load <module>c0 Loads the specified module."
	},
	{
		"unload",
		GameCommandLineUnload,
		"c4.unload <module>c0 Unloads the specified module."
	},
	{
		"list",
		GameCommandLineList,
		"c4.listc0 Lists all currently loaded modules."
	},
	{
		"version",
		GameCommandLineVersion,
		"c4.versionc0 Displays detailed version information on the D2Hackit server dll."
	},
	{
		"credits",
		GameCommandLineCredits,
		"c4.credits"
	},
	{
		"set",
		GameCommandLineSet,
		"c4.set verbose <on|off>c0 Verbose output (Can also be set in D2HackIt.ini)"
	},
	{
		"send",
		GameCommandLineSend,
		"c4.send <packet>c0 Sends a packet to the server. Great for testing stuff :)\n"
		"The packet is not validated so please; no spaces, only hexadecimal number and even number of chars!\n"
		"Try c4'.send 500100000001000000'c0 in a game you created."
	},
	{
		"receive",
		GameCommandLineReceive,
		"c4.receive <packet>c0 Fakes a packet from the server to the game. Great for testing stuff :)\n"
		"The packet is not validated so please; no spaces, only hexadecimal number and even number of chars!"
	},
	/*	
	{
		"sendkey",
		GameCommandLineSendkey,
		"c4.sendkey <keycode>c0 Sends a keystroke to the game"
	},
	*/
	{
		"say",
		GameCommandLineSay,
		"c4.say <text>c0 Sends a string of text to the game as if your character said it."
	},
	{
		"overhead",
		GameCommandLineOverhead,
		"c4.overhead <text>c0 Puts a box of text over your head."
	},
	{
		"repeat",
		GameCommandLineRepeat,
		"c4.repeat <count> <command>c0 Repeats <command>, <count> number of times"
	},
	/*
	{
		"test",
		GameCommandLineTest,
		"c4.testc0 This does nothing..."
	},
	*/
	{
		"crash",
		GameCommandLineCrash,
		"c4.crashc0 This crashes the game in those rare cases when you *can't* exit a game!"
	},

	{
		NULL,
	}
};

//////////////////////////////////////////////////////////////////////
// FindGameCommand
// -------------------------------------------------------------------
// Returns pointer to a COMMANDSTRUCT
//////////////////////////////////////////////////////////////////////
GAMECOMMANDSTRUCT* PRIVATE FindGameCommand(char* name)
{
	GAMECOMMANDSTRUCT* gcs = GameCommands;
	while (gcs->szName) {
		if (!stricmp(gcs->szName, name))
			return gcs;
		gcs++;
	}
	return NULL;
}

//////////////////////////////////////////////////////////////////////
// GameCommandLine
// -------------------------------------------------------------------
// This is where we take a command line, tokenize it and sends it to
// the right destination :)
//////////////////////////////////////////////////////////////////////
BOOL EXPORT GameCommandLine(char* raw)
{
	 char*     p; 
     char*     Next_Command; 
 
     // ClckWcher 
     if(Next_Command = strchr(raw,'|')) 
     { 
          *Next_Command = 0; 
          Next_Command ++; 
          GameCommandLine(Next_Command); 
     } 
 
     char buf[1024];
	 
	 // First lets parse for a command line
	char	*t,*msg;
	char*	argv[1024];
	int		argc = 0;

	sprintf(buf, "%s", raw);

	GAMECOMMANDSTRUCT*	gcs;

	argv[argc++]=p=buf;

	// This is druttis' tokenizer :)
	//
	//	Loopa cmdline pekare till null
	while (*p != 0) {
		if (*p == ' ') {
			//
			//	Space, avsluta detta token
			*(p++) = 0;
			//
			//	Unspacea
			while (*p == ' ')
				p++;
			//
			//	Iev, nytt token.
			if (*p != 0)
				argv[argc++] = p;
		} else if (*p == '"') {
			//
			//	Quote hittat,
			*(p++) = 0;
			argv[argc - 1] = p;
			//
			//	Loopa ut quote eller slut
			while ((*p != '"') && (*p != 0))
				p++;
			//
			//	
			if (*p != 0) {
				*(p++) = 0;
				while (*p == ' ')
					p++;
			}
			if (*p != 0)
				argv[argc++] =  p;
			} else
			p++;
	}
	gcs = FindGameCommand(argv[0]);
	//
	// Is this a built-in function ?
	if (gcs) {
		//
		// If functions returns false, show usage help
		if (!gcs->pFunc(argv, argc)) {
			t=new char[strlen(gcs->szUsage)+1];
			fep->GamePrintInfo("Usage:");
			sprintf(t, "%s", gcs->szUsage);
			if (strlen(t))
			{
				msg=p=t;
				while (*p != 0) {
					if (*p == '\n') {
						*(p++) = 0;
						fep->GamePrintInfo(msg);
					if (*p != 0)
						msg = p;
					} else
						p++;
					}
				fep->GamePrintInfo(msg);
			}
			delete t;
		}
	} else {	
		//
		// This is not a built in function
		// Do we have a module loaded with that name ?
		if (!ClientList.GetItemCount())
		{
			// We have no modules...
			fep->GamePrintError("c0Unknown command. Type .help to get help.");
			return FALSE;
		}

		LinkedItem *li=ClientList.GetFirstItem();
		CLIENTINFOSTRUCT* cds;
		for(int i=0; i<ClientList.GetItemCount(); i++)
		{	
			cds=(CLIENTINFOSTRUCT*)li->lpData;
			if (!stricmp(argv[0], cds->Name))
			{
				// If module supports CLI, route the arguments to it
				if (cds->OnGameCommandLine)
					return cds->OnGameCommandLine(argv, argc);
				else
				{
					//
					// If not, just let the user know
					t=new char[128];
					sprintf(t, "Module c4'%s'c0 does not support the command line interface.", argv[0]);
					fep->GamePrintInfo(t);
					delete t;
					return FALSE;
				}
			}
			li=ClientList.GetNextItem(li);
		}
		fep->GamePrintError("c0Unknown command. Type .help to get help.");
	}

     if(Next_Command) 
     { 
          Next_Command --; 
          * Next_Command = '|'; 
     } 
	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// GameCommandLineHelp
// -------------------------------------------------------------------
// Handles command line help
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE GameCommandLineHelp(char** argv, int argc)
{
	if (argc>1) return FALSE;
	//
	// Header
	fep->GamePrintInfo("Available D2HackIt commands:");
	//
	// Loop through commands and write them to screen.
	char t[128];
	for (int i=0; GameCommands[i].szName != NULL; i++)
	{
		sprintf(t, "c4.%s",GameCommands[i].szName);
		fep->GamePrintInfo(t);
	}
	//
	// The two extended modes of '.help'
	fep->GamePrintInfo("c4.<command> help");
	fep->GamePrintInfo("c4.<module> help");
	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// GameCommandLineVersion
// -------------------------------------------------------------------
// 
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE GameCommandLineVersion(char** argv, int argc)
{
	//
	// Show usage if we send too many arguments
	if (argc>1) return FALSE;
	//
	// Just print it :)
	char t[256];
	sprintf(t, "c4D2HackIt! Mk2c0 version %d.%.2d (%s@%s)",LOWORD(si->Version), HIWORD(si->Version), psi->BuildDate, psi->BuildTime);
	fep->GamePrintInfo(t);
	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// GameCommandLineCredits
// -------------------------------------------------------------------
// 
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE GameCommandLineCredits(char** argv, int argc)
{
	//
	// Show usage if we send too many arguments
	if (argc>1) return FALSE;
	//
	// Just print it :)
	fep->GamePrintInfo("CREDITS:");
	fep->GamePrintInfo("These people have in one way or another made this project possible:");
	fep->GamePrintInfo("c4Druttis, mousepad, TechWarrior, Nyttzy99, BlackBird, hello2k1, Arsenic,");
	fep->GamePrintInfo("c4onlyer, icky, RazorBladeEnema, Skull, proff, ClckWtchr");
	fep->GamePrintInfo("BIG thanks to all the beta testers, too many to fit here! :)");
	fep->GamePrintInfo("Also, thanks to d2network for hosting D2HackIt! c4(http://thohell.d2network.com)");
	fep->GamePrintInfo("c4`Socc0 wanted to be mentioned here, so I will mention him as well :)");
	return TRUE;
}


//////////////////////////////////////////////////////////////////////
// GameCommandLineLoad
// -------------------------------------------------------------------
// Loads a client
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE GameCommandLineLoad(char** argv, int argc)
{
	if (argc != 2) return FALSE;
	if (!stricmp(argv[1], "help")) return FALSE;

	char t[1024];


	// Check if module is loaded
	LinkedItem *li=ClientList.GetFirstItem();
	CLIENTINFOSTRUCT* cds;
	for(int i=0; i<ClientList.GetItemCount(); i++)
	{	
		cds=(CLIENTINFOSTRUCT*)li->lpData;
		if (!stricmp(argv[1], cds->Name))
		{
			sprintf(t, "Module '%s' is already loaded!", argv[1]);
			fep->GamePrintError(t);
			return TRUE;
		}
		li=ClientList.GetNextItem(li);
	}

	// Try to load .d2h
	sprintf(t, "%s\\%s.d2h", si->PluginDirectory, argv[1]);
	HMODULE hModule = (HMODULE)LoadLibrary(t);
	if (!hModule)
	{
		// Try to load dll
		sprintf(t, "%s\\%s.d2h", si->PluginDirectory, argv[1]);
		hModule = (HMODULE)LoadLibrary(t);
		if (!hModule)
		{
			// Didn't find any
			sprintf(t, "Unable to load module '%s'!", argv[1]);
			fep->GamePrintError(t);
			return TRUE;
		}
	}


	cds=new CLIENTINFOSTRUCT;
	
	cds->hModule=hModule;
	cds->Name=new char[strlen(argv[1])+1];
	sprintf(cds->Name, "%s", argv[1]);
	
	// Bind event handlers
	BIND(OnGamePacketBeforeSent);
	BIND(OnGamePacketBeforeReceived);
	BIND(OnGameCommandLine);
	BIND(OnGameJoin);
	BIND(OnGameLeave);
	BIND(OnGameTimerTick);
	BIND(OnClientStart);
	BIND(OnClientStop);
	
	// Bind property gets
	BIND(GetModuleAuthor);
	BIND(GetModuleWebsite);
	BIND(GetModuleEmail);
	BIND(GetModuleDescription);
	BIND(GetModuleVersion);

	ClientList.AddItem(cds);

	// Initiate client
	if ((cds->OnClientStart))
	{
		// If initialization fails, unload module
		if (!cds->OnClientStart())
		{
			argv[1]=new char[10];
			sprintf(argv[1], "unload");
			GameCommandLineUnload(argv, argc);
			delete argv[1];
			return TRUE;
		}
	}
	if ((cds->OnGameJoin) && (thisgame->player))
		cds->OnGameJoin(thisgame);

	sprintf(t, "Module c4'%s'c0%s%s loaded!",
		cds->Name,
		(cds->GetModuleAuthor?" by ":""),
		(cds->GetModuleAuthor?cds->GetModuleAuthor():"")
		);
	fep->GamePrintInfo(t);
	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// GameCommandLineUnload
// -------------------------------------------------------------------
// Unloads a client
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE GameCommandLineUnload(char** argv, int argc)
{
	if (argc != 2) return FALSE;
	if (!stricmp(argv[1], "help")) return FALSE;

	if (!ClientList.GetItemCount())
	{
		fep->GamePrintInfo("No modules loaded!");
		return TRUE;
	}

	char t[128];
	LinkedItem *li=ClientList.GetFirstItem();
	CLIENTINFOSTRUCT* cds;
	for(int i=0; i<ClientList.GetItemCount(); i++)
	{	
		cds=(CLIENTINFOSTRUCT*)li->lpData;
		if (!stricmp(argv[1], cds->Name))
		{
			// Tell module that we leave game
			if ((cds->OnGameLeave) && (thisgame->player))
					cds->OnGameLeave(thisgame);

			// Prepare module to be unloaded
			if ((cds->OnClientStop))
			{
				// If this fails, let the user know
				if (!cds->OnClientStop())
				{
					fep->GamePrintInfo("Module not unloaded cleanly! This could ba a problem!");
				}
			}

			FreeLibrary((HMODULE)cds->hModule);
			sprintf(t, "Module c4'%s'c0 unloaded!", argv[1]);
			fep->GamePrintInfo(t);
			ClientList.RemoveItem(li);
			delete cds->Name;
			delete cds;
			return TRUE;
		}
		li=ClientList.GetNextItem(li);
	}
	sprintf(t, "Could not unload module '%s'. It's not loaded!",argv[1]);
	fep->GamePrintError(t);
	return TRUE;
}


//////////////////////////////////////////////////////////////////////
// GameCommandLineList
// -------------------------------------------------------------------
// Lists loaded clients
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE GameCommandLineList(char** argv, int argc)
{
	//
	// Show usage if we send too many arguments
	if (argc>1) return FALSE;

	//
	// Are any modules loaded ?
	if (!ClientList.GetItemCount())
	{
		fep->GamePrintInfo("No modules loaded!");
		return TRUE;
	}
	//
	// Header
	fep->GamePrintInfo("Loaded modules:");
	//
	// Loop through modules and print them to screen
	LinkedItem *li=ClientList.GetFirstItem();
	CLIENTINFOSTRUCT* cds;
	for(int i=0; i<ClientList.GetItemCount(); i++)
	{	
		cds=(CLIENTINFOSTRUCT*)li->lpData;
		char t[256];
		sprintf(t, "c4%sc0", cds->Name);
		if (cds->GetModuleVersion)
			sprintf(t, "%s version %d.%d",t, LOWORD(cds->GetModuleVersion()), HIWORD(cds->GetModuleVersion()));
		if (cds->GetModuleAuthor)
			sprintf(t, "%s by %s", t, cds->GetModuleAuthor());
		if (cds->GetModuleEmail)
			sprintf(t, "%s (%s)", t, cds->GetModuleEmail());
		if (cds->GetModuleWebsite)
			sprintf(t, "%s c4http://%s", t, cds->GetModuleWebsite());

		fep->GamePrintInfo(t);
		li=ClientList.GetNextItem(li);
	}
	return TRUE;
}


//////////////////////////////////////////////////////////////////////
// GameCommandLineSet
// -------------------------------------------------------------------
// Set server options
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE GameCommandLineSet(char** argv, int argc)
{
	// Too few arguments, show usage
	if (argc<3) return FALSE;
	//
	// Check for 'verbose' options
	if (!stricmp(argv[1], "verbose"))
	{
		if (!stricmp(argv[2], "on"))
		{
			psi->Verbose = TRUE;
			fep->GamePrintInfo("Verbose operation is c4ON");
			SetHackProfileString("D2HackIt", "Misc", "Verbose", "on");
			return TRUE;
		}
		if (!stricmp(argv[2], "off"))
		{
			psi->Verbose = FALSE;
			fep->GamePrintInfo("Verbose operation is c4OFF");
			SetHackProfileString("D2HackIt", "Misc", "Verbose", "off");
			return TRUE;
		}
	}
	//
	// No valid options found, show usage
	return FALSE;
}

//////////////////////////////////////////////////////////////////////
// GameCommandLineSend
// -------------------------------------------------------------------
// QnD packet sender for testing stuff
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE GameCommandLineSend(char** argv, int argc)
{
	//
	// Some sanity checks
	if (argc != 2) return FALSE;
	if (strlen(argv[1])%2) return FALSE;
	if (!stricmp(argv[1], "help")) return FALSE;
	//
	// Make a buffer
	int len=strlen(argv[1])/2;
	PBYTE buf=new BYTE[len];
	//
	// Build the packet
	char *x="";
	for (int i=len-1; strlen(argv[1]) > 0; i--)
	{
		buf[i]=(BYTE)strtoul(&argv[1][i*2], &x, 0x10);
		argv[1][i*2]=0;
	}
	//
	// Send the packet
	GameSendPacketToServer(buf, len);
	//
	// Done
	delete buf;
	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// GameCommandLineReceive
// -------------------------------------------------------------------
// QnD incoming packet sender for testing stuff
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE GameCommandLineReceive(char** argv, int argc)
{
	//
	// Some sanity checks
	if (argc != 2) return FALSE;
	if (strlen(argv[1])%2) return FALSE;
	if (!stricmp(argv[1], "help")) return FALSE;
	//
	// Make a buffer
	int len=strlen(argv[1])/2;
	PBYTE buf=new BYTE[len];
	//
	// Build the packet
	char *x="";
	for (int i=len-1; strlen(argv[1]) > 0; i--)
	{
		buf[i]=(BYTE)strtoul(&argv[1][i*2], &x, 0x10);
		argv[1][i*2]=0;
	}
	//
	// Send the packet
	GameSendPacketToGame(buf, len);
	//
	// Done
	delete buf;
	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// GameCommandLineReceive
// -------------------------------------------------------------------
// QnD incoming packet sender for testing stuff
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE GameCommandLineSendkey(char** argv, int argc)
{
	// Sanity checks
	if (argc != 2) return FALSE;
	if (strlen(argv[1]) != 2) return FALSE;
	char* x="";
	DWORD Keycode=strtoul(argv[1], &x, 0x10);
	DWORD p = psi->GameKeyDownIntercept;

	fep->GamePrintError(".sendkey is not working yet!");

	/*
	__asm {
		mov esi,0x20
		call p
	}
	*/

	return TRUE;
}


//////////////////////////////////////////////////////////////////////
// GameCommandLineSay
// -------------------------------------------------------------------
// Makes your character say something in game
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE GameCommandLineSay(char** argv, int argc)
{
	// Sanity check
	if (argc<2) return FALSE;
		
	BYTE aPacket[1024];
	char aString[1024];
	aString[0]=0;
	sprintf(aString, "%s", argv[1]);

	for (int i=2; i!=argc; i++)
	{
		sprintf(aString, "%s %s", aString, argv[i]);
	}

	sprintf((char*)aPacket, "%c%c%c%s%c%c%c",
			0x15, 0x01, 0x00, aString, 0,0,0);
	GameSendPacketToServer(aPacket, strlen(aString)+6);

	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// GameCommandLineOverhead
// -------------------------------------------------------------------
// Makes your character say something in game over his head.
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE GameCommandLineOverhead(char** argv, int argc)
{
	// Sanity check
	if (argc<2) return FALSE;
		
	BYTE aPacket[1024];
	char aString[1024];
	aString[0]=0;
	sprintf(aString, "%s", argv[1]);

	for (int i=2; i!=argc; i++)
	{
		sprintf(aString, "%s %s", aString, argv[i]);
	}

	sprintf((char*)aPacket, "%c%c%c%s%c%c%c",
			0x14, 0x00, 0x00, aString, 0,0,0);
	GameSendPacketToServer(aPacket, strlen(aString)+6);

	return TRUE;
}


//////////////////////////////////////////////////////////////////////
// GameCommandLineRepeat
// -------------------------------------------------------------------
// Repeats <command>, <count> number of times
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE GameCommandLineRepeat(char** argv, int argc)
{
	if (argc < 3)
		return FALSE;

	int count=atoi(argv[1]);
	
	// Build commandline
	char t[256];
	t[0]=0;
	for(int i=2;i!=argc;i++)
		sprintf(t, "%s %s", t, argv[i]);
	
	fep->GamePrintInfo(t);

	// Send command
	for (; count; count--)
		GameCommandLine(t+1);

	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// GameCommandLineCrash
// -------------------------------------------------------------------
// Crashes game, not recommended for normal use :)
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE GameCommandLineCrash(char** argv, int argc)
{
	// Jump to nowhere should do it :)
	__asm
	{
		push 0
		ret
	}
	return TRUE;
}

	

//////////////////////////////////////////////////////////////////////
// GameCommandLineTest
// -------------------------------------------------------------------
// Test stuff goes here
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE GameCommandLineTest(char** argv, int argc)
{
	fep->GamePrintInfo("c1Test does nothing!");
	return TRUE;
}

	
